#! /usr/bin/env node
const AWS = require('aws-sdk');
const fs = require('fs-extra');
var async = require('async');
const dotenv = require('dotenv');
dotenv.config()

const defaultBuildlist = './lambda.buildlist';

exports.default = function() {
  const args = process.argv.slice(2);
  if (args.length == 0) {
    if (fs.existsSync(defaultBuildlist)) {
      startBuild(defaultBuildlist)
    }
    else {
      console.error('You must provide a lambda.buildlist file');
      process.exit(1); //an error occurred
    }
  }
  else if (args.length == 1) {
    startBuild(args[0]);
  }
  else {
    console.log(`Could not determine ${defaultBuildlist}`);
    process.exit(1); //no errors occurred
  }
}

function startBuild(buildlist) {
  console.log(`Buildlist detected: ${buildlist}`);
  async.waterfall([
    async.apply(readBuildList, buildlist), 
    createFileList,
    zipFiles,
    lambdaUpdate
  ], function(err) {
    if (err) { console.log('Waterfall failed') }
    else { console.log('All tasks complete') }
  })
}

function readBuildList(buildlist, callback){
  fs.readFile(buildlist, function(err, data) {
    if(err) throw err;
    let buildarray = data.toString().replace(/\r/g, '').split('\n')
    callback(null, buildarray);
  });
}

function createFileList(buildarray, createFileListCallback){
  const glob = require('glob');
  let fileList = {}

  // console.log(buildarray);
  async.each(
    buildarray, function(builditem, nextItem) {
      glob(builditem, function (er, files) {
        async.each(files, function(file, nextfile){

          if (fs.statSync(file).isDirectory()) { nextfile(); }
          else {
            fs.readFile(file, function(err, fileContent) {
              if(err) { return nextfile(err)};
              fileList[file] = fileContent;
              nextfile();
            });
          }
        }, function(err) { 
          if (err) throw err;
          nextItem();
        })
      });
    },
    function(err) {
      console.log(fileList);
      createFileListCallback(null, fileList);
    }
  );
}

function zipFiles(fileList, callback){
  var JSZip = require("jszip");
  var zip = new JSZip();

  async.forEachOf(fileList, function(fileForZip, key, filesAdded) {
    zip.file(key, fileForZip);
    filesAdded();
  });

  zip.generateAsync({
    streamFiles:true,
    type: 'arraybuffer'
  }).then(function (zipData) { 
    callback(null, zipData);
  });

  
}

function lambdaUpdate(zipData, callback){
  if (!process.env.AWS_LAMBDA_NAME) {
    console.log('Cannot find .env');
    process.exit(1)
  }

  var lambda = new AWS.Lambda({region: process.env.AWS_REGION});
  var params = {
    FunctionName: process.env.AWS_LAMBDA_NAME,
    ZipFile: zipData,
    Publish: true,
  };
  lambda.updateFunctionCode(params, function(err, data) {
    if (err) console.log(err, err.stack);
    console.log(`## SUCCESS ## Lambda updated: ${process.env.AWS_LAMBDA_NAME}`)
  });
  
  console.log('pushing: ' + zipData);
  callback(null);
}